from markdown.extensions.fenced_code import FencedBlockPreprocessor
import markdown


class CustomFencedBlockPreprocessor(FencedBlockPreprocessor):
    CODE_WRAP = '<pre%s><code>%s</code></pre>'
    LANG_TAG = ' class="highlight %s"'


class FencedCodeExtension(markdown.Extension):

    def extendMarkdown(self, md, md_globals):
        """ Add FencedBlockPreprocessor to the Markdown instance. """
        md.registerExtension(self)

        md.preprocessors.add('fenced_code_block',
                             CustomFencedBlockPreprocessor(md),
                             ">normalize_whitespace")


def render_markdown(md_text):
    return markdown.markdown(md_text, extensions=[FencedCodeExtension(), "tables"])



# coding: utf-8
from coredocs.highlighting import get_highlight_css, get_all_styles
from coredocs.render_markdown import render_markdown
from coredocs.utils import get_links, get_sections
import coreapi
import coredocs
import click
import jinja2
import os
import shutil
import sys
import pkg_resources


THEME_CHOICES = ['slate', 'cerulean']
PYGMENTS_STYLES = get_all_styles()
DEFAULT_LANGS = ['shell', 'javascript', 'python']


def get_fields(location, link):
    return [field for field in link.fields if field.location == location]


def render(doc, highlight=None, langs=None, theme=None, static=None):
    if static is None:
        static = lambda path: path

    if langs is None:
        langs = DEFAULT_LANGS

    schema_format = determine_format(doc)

    base_templates = os.path.join('themes', 'base', 'templates')
    theme_templates = os.path.join('themes', theme, 'templates')

    env = jinja2.Environment(
        loader=jinja2.ChoiceLoader([
            jinja2.PackageLoader('coredocs', theme_templates),
            jinja2.PackageLoader('coredocs', base_templates)
        ]),
        extensions=['coredocs.highlighting.HighlightExtension']
    )
    template = env.get_template('index.html')
    return template.render(
        static=static,
        render_markdown=render_markdown,
        document=doc,
        get_sections=get_sections,
        get_links=get_links,
        get_fields=get_fields,
        code_style=get_highlight_css(highlight),
        schema_url=doc.url,
        langs=langs,
        schema_format=schema_format
    )


def determine_format(document):
    codecs = get_installed_codecs()
    for format_name, codec in codecs.items():
        if codec.media_type == document.media_type:
            return format_name
    return None


def get_installed_codecs():
    packages = [
        (package, package.load()) for package in
        pkg_resources.iter_entry_points(group='coreapi.codecs')
    ]
    return {
        package.name: cls() for (package, cls) in packages
        if '+' in getattr(cls, 'media_type', '') and hasattr(cls, 'decode')
    }


def get_client(format=None):
    codecs = get_installed_codecs()
    if format is None:
        decoders = list(codecs.values())
    else:
        decoders = [codecs[format]]
    return coreapi.Client(decoders=decoders)


@click.group(invoke_without_command=True, help='CoreDocs. An API documentation generator.')
@click.option('--version', is_flag=True, help='Display the `coredocs` version number.')
@click.pass_context
def client(ctx, version):
    if ctx.invoked_subcommand is not None:
        return

    if version:
        click.echo('coredocs %s' % client_version)
    else:
        click.echo(ctx.get_help())


@click.command(help='Build API documentation.')
@click.argument('url')
@click.option('--format', default=None, help='The format of the input API schema.')
@click.option('--theme', default='slate', help='The theme to use.', type=click.Choice(THEME_CHOICES))
@click.option('--highlight', default='monokai', help='Select a code highlighting style from the pygments library.', type=click.Choice(PYGMENTS_STYLES))
@click.option('--languages', default=','.join(DEFAULT_LANGS), help='List of languages to include, as a comma seperated list.')
@click.option('--outdir', type=click.Path(dir_okay=True), default='build', help='The output directory to use.')
@click.option('--force', help='Delete and overwrite any existing build directory.', is_flag=True)
def build(url, format, theme, highlight, languages, outdir, force):
    if os.path.exists(outdir) and not force:
        click.echo(
            "Output directory '%s' already exists.\n"
            "Delete the directory first, or use the '--force' flag." % outdir
        )
        sys.exit(1)

    if languages:
        languages = languages.split(',')
    else:
        languages = []

    force_codec = format is not None
    client = get_client(format)

    click.echo("Fetching schema file from '%s'." % url)
    try:
        document = client.get(url, force_codec=force_codec)
    except coreapi.exceptions.NoCodecAvailable:
        click.echo(
            "The returned Content-Type does not correspond to any registered schema type.\n"
            "Use the '--format' flag to explicitly select an input schema type."
        )
        sys.exit(1)
    except coreapi.exceptions.ErrorMessage as exc:
        click.echo(display(exc.error))
        sys.exit(1)

    output_text = render(document, highlight, languages, theme=theme)

    base_dir = os.path.dirname(coredocs.__file__)
    static_dir = os.path.join(base_dir, 'themes', theme, 'static')

    click.echo("Building documentation into directory '%s'." % outdir)
    if os.path.exists(outdir) and force:
        shutil.rmtree(outdir)
    shutil.copytree(static_dir, os.path.join(outdir))

    output_file = os.path.join(outdir, 'index.html')
    with open(output_file, 'w') as output_file:
        output_file.write(output_text)


client.add_command(build)


if __name__ == '__main__':
    client()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re
import os
import sys


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = get_version('coredocs')


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


setup(
    name='coredocs',
    version=version,
    url='http://github.com/core-api/coredocs/',
    license='BSD',
    description='API documentation generator.',
    author='Tom Christie',
    author_email='tom@tomchristie.com',
    packages=get_packages('coredocs'),
    package_data=get_package_data('coredocs'),
    install_requires=[
        'coreapi>=2.0.0',
        'click>=6.0',
        'jinja2>=2.8',
        'Markdown>=2.6.0',
        'Pygments>=2.1.0'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    entry_points={
        'console_scripts': [
            'coredocs=coredocs.main:client'
        ],
    },
)



def link_sort(link_item):
    key, link = link_item
    action_priority = {
        '': 0, 'get': 0,
        'post': 1,
        'put': 2,
        'patch': 3,
        'delete': 4
    }.get(link.action, 5)
    return (link.url, action_priority)


def get_sections(node):
    return sorted(node.data.iteritems(), key=lambda item: item[0])


def get_links(node):
    return sorted(node.links.iteritems(), key=link_sort)




from jinja2 import nodes
from jinja2.ext import Extension, Markup

from pygments import highlight, styles
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter


DEFAULT_STYLE = 'monokai'


class HighlightExtension(Extension):
    """
    Code block highlighting.

        {% highlight 'python' %}
        print("hello, world")
        {% endhighlight %}
    """
    tags = set(['highlight'])
    style = DEFAULT_STYLE

    def parse(self, parser):
        next(parser.stream)
        args = [parser.parse_expression()]
        body = parser.parse_statements(['name:endhighlight'], drop_needle=True)
        return nodes.CallBlock(self.call_method('_highlight', args), [], [], body)

    def _highlight(self, lang, caller=None):
        body = caller()
        lexer = get_lexer_by_name(lang, stripall=False)
        formatter = HtmlFormatter(nowrap=True, style=self.style)
        code = highlight(Markup(body.rstrip()).unescape(), lexer, formatter)
        return code


def get_highlight_css(style=None):
    if style is None:
        style = DEFAULT_STYLE
    return HtmlFormatter(style=style).get_style_defs('.highlight')

def get_all_styles():
    return sorted(list(styles.get_all_styles()))
